@@ -5,14 +5,12 @@
 use std::collections::HashMap;
 use std::io::Write;
 
-use syn;
-
 use crate::bindgen::config::{Config, Language};
 use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
 use crate::bindgen::dependencies::Dependencies;
 use crate::bindgen::ir::{
-    AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, Item, ItemContainer, Path,
-    ToCondition, Type,
+    AnnotationSet, Cfg, ConditionWrite, Documentation, Field, GenericParams, Item, ItemContainer,
+    Path, ToCondition, Type,
 };
 use crate::bindgen::library::Library;
 use crate::bindgen::mangle;
@@ -68,8 +66,8 @@ impl Typedef {
         }
     }
 
-    pub fn simplify_standard_types(&mut self) {
-        self.aliased.simplify_standard_types();
+    pub fn simplify_standard_types(&mut self, config: &Config) {
+        self.aliased.simplify_standard_types(config);
     }
 
     pub fn transfer_annotations(&mut self, out: &mut HashMap<Path, AnnotationSet>) {
@@ -140,6 +138,10 @@ impl Item for Typedef {
         self.aliased.rename_for_config(config, &self.generic_params);
     }
 
+    fn collect_declaration_types(&self, resolver: &mut DeclarationTypeResolver) {
+        resolver.add_none(&self.path);
+    }
+
     fn resolve_declaration_types(&mut self, resolver: &DeclarationTypeResolver) {
         self.aliased.resolve_declaration_types(resolver);
     }
@@ -177,7 +179,7 @@ impl Item for Typedef {
         let mangled_path = mangle::mangle_path(
             &self.path,
             generic_values,
-            library.get_config().export.mangle_separator.as_deref(),
+            &library.get_config().export.mangle,
         );
 
         let monomorph = Typedef::new(
@@ -205,13 +207,18 @@ impl Source for Typedef {
 
         self.generic_params.write(config, out);
 
-        if config.language == Language::C {
-            out.write("typedef ");
-            (self.export_name().to_owned(), self.aliased.clone()).write(config, out);
-        } else {
-            write!(out, "using {} = ", self.export_name());
-            self.aliased.write(config, out);
+        match config.language {
+            Language::Cxx => {
+                write!(out, "using {} = ", self.export_name());
+                self.aliased.write(config, out);
+            }
+            Language::C | Language::Cython => {
+                write!(out, "{} ", config.language.typedef());
+                Field::from_name_and_type(self.export_name().to_owned(), self.aliased.clone())
+                    .write(config, out);
+            }
         }
+
         out.write(";");
 
         condition.write_after(config, out);
