@@ -8,8 +8,6 @@ use std::fs::File;
 use std::io::Read;
 use std::path::{Path as FilePath, PathBuf as FilePathBuf};
 
-use syn;
-
 use crate::bindgen::bitflags;
 use crate::bindgen::cargo::{Cargo, PackageRef};
 use crate::bindgen::config::{Config, ParseConfig};
@@ -98,7 +96,7 @@ struct Parser<'a> {
 }
 
 impl<'a> Parser<'a> {
-    fn should_parse_dependency(&self, pkg_name: &String) -> bool {
+    fn should_parse_dependency(&self, pkg_name: &str) -> bool {
         if self.parsed_crates.contains(pkg_name) {
             return false;
         }
@@ -108,20 +106,33 @@ impl<'a> Parser<'a> {
         }
 
         // Skip any whitelist or blacklist for expand
-        if self.config.parse.expand.crates.contains(&pkg_name) {
+        if self
+            .config
+            .parse
+            .expand
+            .crates
+            .iter()
+            .any(|name| name == pkg_name)
+        {
             return true;
         }
 
         // If we have a whitelist, check it
         if let Some(ref include) = self.config.parse.include {
-            if !include.contains(&pkg_name) {
+            if !include.iter().any(|name| name == pkg_name) {
                 debug!("Excluding crate {}", pkg_name);
                 return false;
             }
         }
 
         // Check the blacklist
-        !STD_CRATES.contains(&pkg_name.as_ref()) && !self.config.parse.exclude.contains(&pkg_name)
+        !STD_CRATES.contains(&pkg_name)
+            && !self
+                .config
+                .parse
+                .exclude
+                .iter()
+                .any(|name| name == pkg_name)
     }
 
     fn parse_crate(&mut self, pkg: &PackageRef) -> Result<(), Error> {
@@ -190,6 +201,7 @@ impl<'a> Parser<'a> {
                         self.config.parse.expand.all_features,
                         self.config.parse.expand.default_features,
                         &self.config.parse.expand.features,
+                        self.config.parse.expand.profile,
                     )
                     .map_err(|x| Error::CargoExpand(pkg.name.clone(), x))?;
                 let i = syn::parse_file(&s).map_err(|x| Error::ParseSyntaxError {
@@ -203,7 +215,7 @@ impl<'a> Parser<'a> {
             self.cache_expanded_crate.get(&pkg.name).unwrap().clone()
         };
 
-        self.process_mod(pkg, None, &mod_items, 0)
+        self.process_mod(pkg, None, None, &mod_items, 0)
     }
 
     fn parse_mod(
@@ -238,30 +250,37 @@ impl<'a> Parser<'a> {
         };
 
         // Compute module directory according to Rust 2018 rules
-        let mod_dir_2018;
+        let submod_dir_2018;
 
-        let mod_dir = if depth == 0 || mod_path.ends_with("mod.rs") {
-            mod_path.parent().unwrap()
+        let mod_dir = mod_path.parent().unwrap();
+
+        let submod_dir = if depth == 0 || mod_path.ends_with("mod.rs") {
+            mod_dir
         } else {
-            mod_dir_2018 = mod_path
+            submod_dir_2018 = mod_path
                 .parent()
                 .unwrap()
                 .join(mod_path.file_stem().unwrap());
-            &mod_dir_2018
+            &submod_dir_2018
         };
 
-        self.process_mod(pkg, Some(&mod_dir), &mod_items, depth)
+        self.process_mod(pkg, Some(mod_dir), Some(submod_dir), &mod_items, depth)
     }
 
     /// `mod_dir` is the path to the current directory of the module. It may be
     /// `None` for pre-expanded modules.
+    ///
+    /// `submod_dir` is the path to search submodules in by default, which might
+    /// be different for rust 2018 for example.
     fn process_mod(
         &mut self,
         pkg: &PackageRef,
         mod_dir: Option<&FilePath>,
+        submod_dir: Option<&FilePath>,
         items: &[syn::Item],
         depth: usize,
     ) -> Result<(), Error> {
+        debug_assert_eq!(mod_dir.is_some(), submod_dir.is_some());
         // We process the items first then the nested modules.
         let nested_modules = self.out.load_syn_crate_mod(
             &self.config,
@@ -273,23 +292,24 @@ impl<'a> Parser<'a> {
 
         for item in nested_modules {
             let next_mod_name = item.ident.to_string();
-
             let cfg = Cfg::load(&item.attrs);
             if let Some(ref cfg) = cfg {
                 self.cfg_stack.push(cfg.clone());
             }
 
             if let Some((_, ref inline_items)) = item.content {
-                let next_mod_dir = mod_dir.map(|dir| dir.join(&next_mod_name));
+                let next_submod_dir = submod_dir.map(|dir| dir.join(&next_mod_name));
                 self.process_mod(
                     pkg,
-                    next_mod_dir.as_ref().map(|p| &**p),
+                    mod_dir,
+                    next_submod_dir.as_deref(),
                     inline_items,
                     depth,
                 )?;
             } else if let Some(mod_dir) = mod_dir {
-                let next_mod_path1 = mod_dir.join(next_mod_name.clone() + ".rs");
-                let next_mod_path2 = mod_dir.join(next_mod_name.clone()).join("mod.rs");
+                let submod_dir = submod_dir.unwrap();
+                let next_mod_path1 = submod_dir.join(next_mod_name.clone() + ".rs");
+                let next_mod_path2 = submod_dir.join(next_mod_name.clone()).join("mod.rs");
 
                 if next_mod_path1.exists() {
                     self.parse_mod(pkg, next_mod_path1.as_path(), depth + 1)?;
@@ -299,22 +319,21 @@ impl<'a> Parser<'a> {
                     // Last chance to find a module path
                     let mut path_attr_found = false;
                     for attr in &item.attrs {
-                        match attr.parse_meta() {
-                            Ok(syn::Meta::NameValue(syn::MetaNameValue { path, lit, .. })) => {
-                                match lit {
-                                    syn::Lit::Str(ref path_lit) if path.is_ident("path") => {
-                                        path_attr_found = true;
-                                        self.parse_mod(
-                                            pkg,
-                                            &mod_dir.join(path_lit.value()),
-                                            depth + 1,
-                                        )?;
-                                        break;
-                                    }
-                                    _ => (),
+                        if let Ok(syn::Meta::NameValue(syn::MetaNameValue { path, lit, .. })) =
+                            attr.parse_meta()
+                        {
+                            match lit {
+                                syn::Lit::Str(ref path_lit) if path.is_ident("path") => {
+                                    path_attr_found = true;
+                                    self.parse_mod(
+                                        pkg,
+                                        &mod_dir.join(path_lit.value()),
+                                        depth + 1,
+                                    )?;
+                                    break;
                                 }
+                                _ => (),
                             }
-                            _ => (),
                         }
                     }
 
@@ -358,13 +377,13 @@ pub struct Parse {
 impl Parse {
     pub fn new() -> Parse {
         Parse {
-            constants: ItemMap::new(),
-            globals: ItemMap::new(),
-            enums: ItemMap::new(),
-            structs: ItemMap::new(),
-            unions: ItemMap::new(),
-            opaque_items: ItemMap::new(),
-            typedefs: ItemMap::new(),
+            constants: ItemMap::default(),
+            globals: ItemMap::default(),
+            enums: ItemMap::default(),
+            structs: ItemMap::default(),
+            unions: ItemMap::default(),
+            opaque_items: ItemMap::default(),
+            typedefs: ItemMap::default(),
             functions: Vec::new(),
         }
     }
@@ -397,6 +416,8 @@ impl Parse {
         add_opaque("BTreeSet", vec!["T"]);
         add_opaque("LinkedList", vec!["T"]);
         add_opaque("VecDeque", vec!["T"]);
+        add_opaque("ManuallyDrop", vec!["T"]);
+        add_opaque("MaybeUninit", vec!["T"]);
     }
 
     pub fn extend_with(&mut self, other: &Parse) {
@@ -607,6 +628,7 @@ impl Parse {
         );
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn load_fn_declaration(
         &mut self,
         config: &Config,
@@ -631,7 +653,7 @@ impl Parse {
         }
 
         let loggable_item_name = || {
-            let mut items = vec![];
+            let mut items = Vec::with_capacity(3);
             items.push(crate_name.to_owned());
             if let Some(ref self_type) = self_type {
                 items.push(self_type.to_string());
@@ -908,14 +930,6 @@ impl Parse {
         mod_cfg: Option<&Cfg>,
         item: &syn::ItemEnum,
     ) {
-        if item.generics.lifetimes().count() > 0 {
-            info!(
-                "Skip {}::{} - (has generics or lifetimes or where bounds).",
-                crate_name, &item.ident
-            );
-            return;
-        }
-
         match Enum::load(item, mod_cfg, config) {
             Ok(en) => {
                 info!("Take {}::{}.", crate_name, &item.ident);
