@@ -6,7 +6,7 @@ use std::cmp;
 use std::io;
 use std::io::Write;
 
-use crate::bindgen::config::{Braces, Config};
+use crate::bindgen::config::{Braces, Config, Language};
 use crate::bindgen::Bindings;
 
 /// A type of way to format a list.
@@ -65,6 +65,7 @@ pub struct SourceWriter<'a, F: Write> {
     line_number: usize,
     max_line_length: usize,
 }
+
 pub type MeasureWriter<'a> = SourceWriter<'a, NullFile>;
 
 impl<'a, F: Write> SourceWriter<'a, F> {
@@ -137,7 +138,9 @@ impl<'a, F: Write> SourceWriter<'a, F> {
     }
 
     pub fn new_line(&mut self) {
-        writeln!(self.out).unwrap();
+        self.out
+            .write_all(self.bindings.config.line_endings.as_str().as_bytes())
+            .unwrap();
         self.line_started = false;
         self.line_length = 0;
         self.line_number += 1;
@@ -150,28 +153,40 @@ impl<'a, F: Write> SourceWriter<'a, F> {
     }
 
     pub fn open_brace(&mut self) {
-        match self.bindings.config.braces {
-            Braces::SameLine => {
-                self.write(" {");
+        match self.bindings.config.language {
+            Language::Cxx | Language::C => match self.bindings.config.braces {
+                Braces::SameLine => {
+                    self.write(" {");
+                    self.push_tab();
+                    self.new_line();
+                }
+                Braces::NextLine => {
+                    self.new_line();
+                    self.write("{");
+                    self.push_tab();
+                    self.new_line();
+                }
+            },
+            Language::Cython => {
+                self.write(":");
+                self.new_line();
                 self.push_tab();
-                self.new_line();
-            }
-            Braces::NextLine => {
-                self.new_line();
-                self.write("{");
-                self.push_tab();
-                self.new_line();
             }
         }
     }
 
     pub fn close_brace(&mut self, semicolon: bool) {
         self.pop_tab();
-        self.new_line();
-        if semicolon {
-            self.write("};");
-        } else {
-            self.write("}");
+        match self.bindings.config.language {
+            Language::Cxx | Language::C => {
+                self.new_line();
+                if semicolon {
+                    self.write("};");
+                } else {
+                    self.write("}");
+                }
+            }
+            Language::Cython => {}
         }
     }
 
